@@ -20,6 +20,7 @@
 
 from argparse import ArgumentParser
 from datetime import datetime
+from filelock import FileLock
 from itertools import cycle
 from oc_meta.lib.csvmanager import CSVManager
 from oc_meta.lib.file_manager import get_data, normalize_path, pathoo, suppress_stdout, init_cache, sort_files
@@ -30,9 +31,11 @@ from oc_meta.scripts.curator import Curator
 from multiprocessing.pool import ApplyResult, Pool
 from oc_ocdm import Storer
 from oc_ocdm.prov import ProvSet
+from pathlib import Path
+from shutil import make_archive
+from sys import platform
 from time_agnostic_library.support import generate_config_file
-from tqdm import tqdm
-from typing import List, Set, Tuple
+from typing import List, Set
 import csv
 import os
 import yaml
@@ -42,6 +45,7 @@ class MetaProcess:
     def __init__(self, config:str):
         with open(config, encoding='utf-8') as file:
             settings = yaml.full_load(file)
+        self.config = config
         # Mandatory settings
         self.triplestore_url = settings['triplestore_url']
         self.input_csv_dir = normalize_path(settings['input_csv_dir'])
@@ -88,7 +92,9 @@ class MetaProcess:
         csv.field_size_limit(128)
         return files_to_be_processed
 
-    def curate_and_create(self, filename:str, worker_number:int=None, resp_agents_only:bool=False) -> Tuple[Storer, Storer, str]:
+    def curate_and_create(self, filename:str, cache_path:str, errors_path:str, worker_number:int=None, resp_agents_only:bool=False) -> None:
+        if os.path.exists(os.path.join(self.base_output_dir, '.stop')):
+            return
         try:
             filepath = os.path.join(self.input_csv_dir, filename)
             data = get_data(filepath)
@@ -120,9 +126,25 @@ class MetaProcess:
             prov_storer = Storer(prov, context_map={}, dir_split=self.dir_split_number, n_file_item=self.items_per_file, output_format='json-ld')
             with suppress_stdout():
                 self.store_data_and_prov(res_storer, prov_storer, filename)
-            return {'success': filename}
+            lock = FileLock(f'{cache_path}.lock')
+            with lock:
+                if not os.path.exists(cache_path):
+                    with open(cache_path, 'w', encoding='utf-8') as aux_file:
+                        aux_file.write(filename + '\n')
+                else:
+                    with open(cache_path, 'r', encoding='utf-8') as aux_file:
+                        data = aux_file.read().splitlines()
+                        data.append(filename)
+                        data_sorted = sorted(data, key=lambda filename: int(filename.replace('.csv', '')), reverse=False)
+                    with open(cache_path, 'w', encoding='utf-8') as aux_file:
+                        aux_file.write('\n'.join(data_sorted))
         except Exception as e:
-            return {'error': filename, 'msg': e}
+            template = "An exception of type {0} occurred. Arguments:\n{1!r}"
+            message = template.format(type(e).__name__, e.args)
+            lock = FileLock(cache_path)
+            with lock:
+                with open(errors_path, 'a', encoding='utf-8') as aux_file:
+                    aux_file.write(f'{filename}: {message}' + '\n')
     
     def store_data_and_prov(self, res_storer:Storer, prov_storer:Storer, filename:str) -> None:
         if self.rdf_output_in_chunks:
@@ -138,9 +160,8 @@ class MetaProcess:
             res_storer.upload_all(triplestore_url=self.triplestore_url, base_dir=self.output_rdf_dir, batch_size=100)
 
 def run_meta_process(meta_process:MetaProcess, resp_agents_only:bool=False) -> None:
-    delete_lock_files(base_dirs=[meta_process.output_rdf_dir, meta_process.info_dir])
+    delete_lock_files(base_dir=meta_process.base_output_dir)
     files_to_be_processed = meta_process.prepare_folders()
-    pbar = tqdm(total=len(files_to_be_processed)) if meta_process.verbose else None
     max_workers = meta_process.workers_number
     if max_workers == 0:
         workers = list(range(1, os.cpu_count()))
@@ -149,28 +170,50 @@ def run_meta_process(meta_process:MetaProcess, resp_agents_only:bool=False) -> N
     else:
         multiples_of_ten = {i for i in range(1, max_workers+1) if int(i) % 10 == 0}
         workers = [i for i in range(1, max_workers+len(multiples_of_ten)+1) if i not in multiples_of_ten]
-    with Pool(processes=max_workers, maxtasksperchild=1) as executor:
-        futures:List[ApplyResult] = [executor.apply_async(func=meta_process.curate_and_create, args=(file_to_be_processed, worker_number, resp_agents_only)) for file_to_be_processed, worker_number in zip(files_to_be_processed, cycle(workers))]
-        for future in futures:
-            processed_file = future.get()
-            if 'success' in processed_file:
-                with open(meta_process.cache_path, 'a', encoding='utf-8') as aux_file:
-                    aux_file.write(processed_file['success'] + '\n')
-            elif 'error' in processed_file:
-                with open(meta_process.errors_path, 'a', encoding='utf-8') as aux_file:
-                    aux_file.write(f'{processed_file["error"]}: {processed_file["msg"]}' + '\n')
-            pbar.update() if pbar else None
-    if os.path.exists(meta_process.cache_path):
+    is_unix = platform in {'linux', 'linux2', 'darwin'}
+    generate_gentle_buttons(meta_process.base_output_dir, meta_process.config, is_unix)
+    files_chunks = chunks(list(files_to_be_processed), 1000) if is_unix else [files_to_be_processed]
+    for files_chunk in files_chunks:
+        with Pool(processes=max_workers, maxtasksperchild=1) as executor:
+            futures:List[ApplyResult] = [executor.apply_async(
+                func=meta_process.curate_and_create, 
+                args=(file_to_be_processed, meta_process.cache_path, meta_process.errors_path, worker_number, resp_agents_only)) 
+                for file_to_be_processed, worker_number in zip(files_chunk, cycle(workers))]
+            for future in futures:
+                future.get()
+        delete_lock_files(base_dir=meta_process.base_output_dir)
+        if is_unix and not os.path.exists(os.path.join(meta_process.base_output_dir, '.stop')):
+            save_results(meta_process.base_output_dir)
+    if os.path.exists(meta_process.cache_path) and not os.path.exists(os.path.join(meta_process.base_output_dir, '.stop')):
         os.rename(meta_process.cache_path, meta_process.cache_path.replace('.txt', f'_{datetime.now().strftime("%Y-%m-%dT%H_%M_%S_%f")}.txt'))
-    pbar.close() if pbar else None
-    delete_lock_files(base_dirs=[meta_process.output_rdf_dir, meta_process.info_dir])
+    if not is_unix:
+        delete_lock_files(base_dir=meta_process.base_output_dir)
 
-def delete_lock_files(base_dirs:str) -> None:
-    for base_dir in base_dirs:
-        for dirpath, _, filenames in os.walk(base_dir):
-            for filename in filenames:
-                if filename.endswith('.lock'):
-                    os.remove(os.path.join(dirpath, filename))
+def chunks(lst:list, n:int) -> List[list]:
+    '''Yield successive n-sized chunks from lst.'''
+    for i in range(0, len(lst), n):
+        yield lst[i:i + n]
+
+def delete_lock_files(base_dir:list) -> None:
+    for dirpath, _, filenames in os.walk(base_dir):
+        for filename in filenames:
+            if filename.endswith('.lock'):
+                os.remove(os.path.join(dirpath, filename))
+
+def save_results(output_dirpath:str) -> None:
+    output_dirname = f"meta_output_{datetime.now().strftime('%Y-%m-%dT%H_%M_%S_%f')}"
+    parent_dir = Path(output_dirpath)
+    parent_dir = parent_dir.parent.absolute()
+    make_archive(base_name=output_dirname, format='zip', base_dir=output_dirpath)
+
+def generate_gentle_buttons(dir:str, config:str, is_unix:bool):
+    if os.path.exists(os.path.join(dir, '.stop')):
+        os.remove(os.path.join(dir, '.stop'))
+    ext = 'sh' if is_unix else 'bat'
+    with open (f'gently_run.{ext}', 'w') as rsh:
+        rsh.write(f'python -m oc_meta.lib.stopper -t "{dir}" --remove\npython -m oc_meta.run.meta_process -c {config}')
+    with open (f'gently_stop.{ext}', 'w') as rsh:
+        rsh.write(f'python -m oc_meta.lib.stopper -t "{dir}" --add')
 
 
 if __name__ == '__main__':
